/*
 * File: Main.java 
 * Package: dm.vpi.lekce04 
 * Project: Lekce04 
 * Author : David Martinek
 * Address: Department of Intelligent Systems 
 *          Faculty of Information Technology
 *          Brno University of Technology 
 *          Bozetechova 2, 612 66 Brno, 
 *          Czech Republic
 * E-mail : martinek@fit.vutbr.cz 
 * 
 * Created on 17.3.2005 (c) 2005, David Martinek
 * 
 * This project is provided under conditions of GNU General Public Licence. The
 * full text of GNU GPL in on internet: http://www.gnu.org/licenses/gpl.html
 * 
 * $Id: Main.java,v 1.1 2005/03/17 17:51:29 martinek Exp $
 */

package dm.vpi.lekce04;

import dm.vpi.lekce04.matrix.Matrix;

/**
 * <p>
 * Tda zapouzdujc aplikaci
 * </p>
 * <h3>Balky:</h3>
 * 
 * <pre>
 * package dm.vpi.lekce04;
 * </pre>
 * 
 * <p>
 * Balk zapouzdujc cel tento projekt. Vtinou se nedoporuuje vytvet
 * projekty, kter by nebyly zapouzdeny do balku, protoe potom mohou hrozit
 * problmy pi souasnm pouvn vce aplikac. Balk by ml vytvet
 * jednoznan a nezamniteln jmenn prostor.
 * </p>
 * <p>
 * Nzvy balk mohou tvoit hierarchick struktury, kter odpovdaj adresov
 * struktue na disku. Jednotliv rovn se oddluj tekami. Tento balk se
 * napklad nachz v adresov struktue dm/vpi/lekce04/.
 * </p>
 * <p>
 * Doporuovan zpsob pojmenovvn balk:
 * </p>
 * <ul>
 * <li>dbt na jednoznanost </li>
 * <li>jmna by mla bt krtk </li>
 * <li>v hornch rovnch hierarchie by mla bt znaka sttu, firmy nebo
 * autora, m se zajist nezamnitelnost s jinou aplikac </li>
 * </ul>
 * 
 * <h3>Zpstupovn td a balk:</h3>
 * 
 * <pre>
 * import java.util.Vector; // zpstupnn konkrtn tdy
 * import java.util.*; // zpstupnn celho balku
 * </pre>
 * 
 * <p>
 * Pomoc klovho slova <code>import</code> lze aktulnmu souboru
 * zpstupnit jak samostatn tdy z jinch balk (tdy ze stejnho balku
 * jsou viditeln automaticky), tak i cel balky.
 * </p>
 * <p>
 * Existuje doporuen pouvat zpstupnn celho balku co nejmn. Vsledn
 * kd tdy, kter m zpstupnn cel balk je toti o nco pomalej,
 * protoe * Hierarchie balk slou pro logickou organizaci knihoven a
 * aplikac. peklada i interpret musej vyhledvat kd ve zbyten velk
 * mnoin td.
 * </p>
 * <p>
 * Pozor! I kdy se jeden balk nachz uvnit jinho balku, nen jeho obsah
 * viditeln automaticky. Opt je nutn pout klov slovo <code>import</code>.
 * </p>
 * <p>
 * Hierarchie balk slou pro logickou organizaci knihoven a aplikac. V
 * balku maj bt tdy, kter k sob logicky pat. Pokud je cel aplikace v
 * jedinm balku jenom proto, e si chtl programtor uetit prci, je to
 * patn.
 * </p>
 * 
 * @author David Martinek
 */
public class Main
{
  /**
   * Zpracuje argumenty pkazovho dku.
   * 
   * @param args
   *          Pole s jednotlivmi argumenty pkazovho dku.
   */
  public static void processArgs(String[] args)
  {
    for (int i = 0; i < args.length; i++)
    {
      if (args[i].equals("-h") || args[i].equals("--help"))
      {
        showHelp();
        System.exit(0);
      }
      else if (args[i].equals("-t"))
      {
        showTranspose();
      }
      else if (args[i].equals("-a"))
      {
        showAdd();
      }
      else if (args[i].equals("-ac"))
      {
        showAddConst();
      }
      else if (args[i].equals("-s"))
      {
        showSub();
      }
      else if (args[i].equals("-m"))
      {
        showMult();
      }
      else if (args[i].equals("-mc"))
      {
        showMultConst();
      }
      else
      {
        System.out.println("Pozor! Neznm argument: " + args[i]);
      }
    }
  }

  private static void showMultConst()
  {
    System.out.println("=== Nsoben matice konstantou ===");
    Matrix A = Matrix.createRandomMatrix(3, 4);
    System.out.println("Matice A:");
    A.print(System.out);
    System.out.println("Matice A*5:");
    A.multConst(5).print(System.out);
  }

  private static void showMult()
  {
    System.out.println("=== Nsoben matic ===");
    Matrix A = Matrix.createRandomMatrix(2, 3);
    Matrix B = Matrix.createRandomMatrix(3, 5);
    System.out.println("Matice A:");
    A.print(System.out);
    System.out.println("Matice B:");
    B.print(System.out);
    System.out.println("A*B:");
    A.mult(B).print(System.out);
  }

  private static void showSub()
  {
    System.out.println("=== Odetn matic ===");
    Matrix A = Matrix.createRandomMatrix(3, 4);
    Matrix B = Matrix.createRandomMatrix(3, 4);
    System.out.println("Matice A:");
    A.print(System.out);
    System.out.println("Matice B:");
    B.print(System.out);
    System.out.println("A-B:");
    A.sub(B).print(System.out);
  }

  private static void showAddConst()
  {
    System.out.println("=== Piten konstanty k matici ===");
    Matrix A = Matrix.createRandomMatrix(3, 4);
    System.out.println("Matice A:");
    A.print(System.out);
    System.out.println("Matice A+5:");
    A.addConst(5).print(System.out);

  }

  private static void showAdd()
  {
    System.out.println("=== Souet matic ===");
    Matrix A = Matrix.createRandomMatrix(3, 4);
    Matrix B = Matrix.createRandomMatrix(3, 4);
    System.out.println("Matice A:");
    A.print(System.out);
    System.out.println("Matice B:");
    B.print(System.out);
    System.out.println("A+B:");
    A.add(B).print(System.out);

  }

  private static void showTranspose()
  {
    System.out.println("=== Transponovan matice ===");
    Matrix A = Matrix.createRandomMatrix(3, 4);
    System.out.println("Matice A:");
    A.print(System.out);
    System.out.println("Matice AT:");
    A.transpose().print(System.out);

  }

  /**
   * Zobraz npovdu k programu.
   */
  public static void showHelp()
  {
    System.out.println("Lekce .4 - Demonstrace pouit Javovskch balk a "
        + "vybranch objektovch vlastnost jazyka.");
    System.out.println("Matrix - program pro matematick vpoty s maticemi");
    System.out.println("Autor: David Martinek (c) 2005");
    System.out.println("Pepnae:");
    System.out.println("  -h, --help  Zobraz tuto npovdu.");
    System.out.println("  -t  Demonstruje transpozici matice.");
    System.out.println("  -a  Demonstruje souet dvou matic.");
    System.out.println("  -ac  Demonstruje piten konstanty k matici.");
    System.out.println("  -s  Demonstruje odeten dvou matic.");
    System.out.println("  -m  Demonstruje souin dvou matic.");
    System.out.println("  -mc  Demonstruje nsoben matice konstantou.");
  }

  /**
   * Funkce main obsahuje kd, kter se spout jako prvn pi startu programu.
   * 
   * @param args
   *          Argumenty pkazovho dku.
   */
  public static void main(String[] args)
  {
    processArgs(args);
    Matrix m = new Matrix(2, 2);
    // m.m[1][1] = 20;
  }

}

// end of file Main.java
