/*
 * File: Matrix.java 
 * Package: dm.vpi.lekce04.matrix 
 * Project: Lekce04 
 * Author : David Martinek 
 * Address: Department of Intelligent Systems 
 *          Faculty of Information Technology
 *          Brno University of Technology
 *          Bozetechova 2, 612 66 Brno, 
 *          Czech Republic 
 * E-mail : martinek@fit.vutbr.cz 
 * Created on 17.3.2005 (c) 2005, David Martinek 
 * 
 * This project is provided under conditions of GNU General Public Licence. 
 * The full text of GNU GPL in on internet:
 * http://www.gnu.org/licenses/gpl.html 
 * 
 * $Id: Matrix.java,v 1.1 2005/03/17 17:51:29 martinek Exp $
 */
package dm.vpi.lekce04.matrix;

import java.io.PrintStream;
import java.util.Random;

/**
 * <p>
 * Tda Matrix realizuje abstraktn datov typ dvojrozmrn matice. Zapouzduje
 * nejen data, ale tak legln operace nad nimi.
 * </p>
 * <p>
 * Aby data zstala po celou dobu pouvn v konzistentnm stavu, je dobr se
 * dret zsady, e uivatel tdy se k holm datm dostane vhradn pes
 * rozhran (API) realizovan veejnmi metodami.
 * </p>
 * <p>
 * Tda implementuje rozhran <code>Cloneable</code>, aby bylo mon pekrt
 * metodu clone().
 * </p>
 * 
 * @author David Martinek
 */
public class Matrix
    implements Cloneable
{
  /**
   * Zapouzden data pedstavujc dvojrozmrn pole -- matici.
   */
  protected int[][] m; // inicializace je odsunuta do konstruktoru

  /**
   * <p>
   * Alokuje pam pro pole. Tento kd je odsunut do samostatn metody, protoe
   * jej vyuv vce konstruktor. Kdy v budoucnu vznikne poteba provdt
   * alokaci sloitjm zpsobem, bude mon to provst zde na jedinm mst.
   * Kdyby se stejn kd nachzel ve vce metodch, bylo by obtn takovou
   * pravu provst konzistentn.
   * </p>
   * <p>
   * Metoda je oznaena jako <code>protected</code>, aby ji mohli vyuvat
   * ppadn potomci tto tdy. Zrove ale netvo rozhran tdy, take pro
   * bnho uivatele je neviditeln.
   * </p>
   * 
   * @param rows
   *          Poet dk matice.
   * @param cols
   *          Poet sloupc matice.
   */
  protected void create(int rows, int cols)
  {
    m = new int[rows][cols];
  }

  /**
   * <p>
   * Konstruktor - vol se pomoc opertoru new. Vytv a zrove inicializuje
   * novou instanci tdy - objekt.
   * </p>
   * <p>
   * Nelze vyrobit matici neleglnch rozmr (zporn rozmr), protoe Java
   * hld meze pol (narozdl od C, C++) a v tomto ppad by to vyhodilo
   * vjimku {@link java.lang.NegativeArraySizeException}.
   * </p>
   * 
   * @param rows
   *          Poet dk matice.
   * @param cols
   *          Poet sloupc matice.
   */
  public Matrix(int rows, int cols)
  {
    create(rows, cols);
  }

  /**
   * Koprovac konstruktor, kter vyrob nov objekt se stejnmi hodnotami jako
   * zadan dvojrozmrn pole.
   * 
   * @param m
   *          Referenn dvojrozmrn pole.
   */
  public Matrix(int[][] m)
  {
    if (m.length == 0)
    {
      create(0, 0);
    }
    else
    {
      create(m.length, m[0].length);
      setValue(m);
    }
  }

  /**
   * <p>
   * Vyrob jednotkovou tvercovou matici zadanho rozmru.
   * </p>
   * <p>
   * Statickm metodm, kter vytvej objekty uritch vlastnost se tak k
   * <b>tovrn metody</b>.
   * </p>
   * 
   * @param size
   *          Rozmr vyrbn tvercov matice.
   * @return Vrac referenci na novou matici.
   */
  public static Matrix createEMatrix(int size)
  {
    Matrix mat = new Matrix(size, size);

    for (int i = 0; i < size; i++)
    {
      mat.m[i][i] = 1;
    }

    return mat;
  }

  /**
   * Vyrob matici zadanch rozmr a napln ji nhodnmi hodnotami.
   * 
   * @param rows
   *          Poet dk nov matice.
   * @param cols
   *          Poet sloupc nov matice.
   * @return Vrac referenci na novou matici.
   */
  public static Matrix createRandomMatrix(int rows, int cols)
  {
    Matrix mat = new Matrix(rows, cols);
    Random rand = new Random();
    for (int i = 0; i < rows; i++)
    {
      for (int j = 0; j < cols; j++)
      {
        mat.m[i][j] = rand.nextInt(10);
        if (rand.nextBoolean()) 
          mat.m[i][j] = -mat.m[i][j];
      }
    }
    return mat;
  }

  /**
   * Vrac poet dk matice.
   * 
   * @return Poet dk matice
   */
  public int getRows()
  {
    return m.length;
  }

  /**
   * Vrt poet sloupc matice. Na tto metod je pkn vidt, pro je dobr
   * vytvet k datm pstup pouze pes rozhran tvoen veejnmi metodami.
   * Kdyby ml uivatel pstup pmo k 2D poli, mohl by na kontrolu mez
   * zapomenout.
   * 
   * @return Poet sloupc matice.
   */
  public int getCols()
  {
    if (m.length == 0)
      return 0;
    return m[0].length;
  }

  /**
   * <p>
   * Zkopruje hodnoty zadan matice do sebe. Pokud zadan matice nem stejn
   * rozmry jako aktuln objekt, funkce vyhod vjimku. Tato metoda vol jinou
   * privtn metodu, kter dl vlastn akci.
   * </p>
   * <p>
   * Tato metoda slou jako rozhran smrem k uivateli, proto provd kontrolu
   * parametr - v ppad, e nespluj poadovan podmnky, metoda vyhazuje
   * vjimku. V privtn metod se kvli efektivit tyto kontroly u
   * neprovdj, protoe se pedpokld, e autor tto tdy ve odladil.
   * Veejn metod, kter sama nic nedl se k <b>obalovac metoda</b>.
   * </p>
   * 
   * @param m
   *          Matice, jej hodnoty se budou koprovat.
   * @return Vrac referenci na sebe sama.
   * @throws IncompatibleMatrixException
   */
  public Matrix setValueOf(Matrix m)
      throws IncompatibleMatrixException
  {
    if (this.getRows() != m.getRows() || this.getCols() != m.getCols())
    {
      throw new IncompatibleMatrixException();
    }
    return setValue(m.m);
  }

  /**
   * Zkopruje hodnoty zadan matice do sebe. Z dvodu vy efektivity tato
   * metoda neprodukuje dnou vjimku. Je ale vhodn v tomto ppad pout
   * assert (protoe jde o privtn metodu, meme si to dovolit, u veejn
   * metody musme vyhazovat vjimky - pro asi?).
   * 
   * @param m
   *          Matice, jej hodnoty se budou koprovat.
   * @return Vrac referenci na sebe sama.
   */
  private Matrix setValue(int[][] m)
  {
    assert this.m.length == m.length;
    
    for (int i = 0; i < this.m.length; i++)
    {
      assert this.m[i].length == m[i].length;
      this.m[i] = m[i].clone();
      // for (int j = 0; j < this.m[i].length; j++)
      // {
      // this.m[i][j] = m[i][j];
      // }
    }

    return this;
  }

  /**
   * Pite obsah zadan matice k aktuln matici.
   * 
   * @param m
   *          Pitan matice.
   * @return Vrac referenci na sebe sama.
   * @throws IncompatibleMatrixException
   *           Generuje vjimku v ppad, e matice nejsou rozmrov
   *           kompatibiln.
   */
  public Matrix add(Matrix m)
      throws IncompatibleMatrixException
  {
    if (this.getRows() != m.getRows() || this.getCols() != m.getCols())
    {
      throw new IncompatibleMatrixException();
    }
    return add(m.m);
  }

  /**
   * Pite obsah zadan matice k aktuln matici.
   * 
   * @param m
   *          Pitan matice.
   * @return Vrac referenci na sebe sama.
   */
  private Matrix add(int[][] m)
  {
    assert this.m.length == m.length;
    
    for (int i = 0; i < this.m.length; i++)
    {
      assert this.m[i].length == m[i].length;
      for (int j = 0; j < this.m[i].length; j++)
      {
        this.m[i][j] += m[i][j];
      }
    }

    return this;
  }
  
  /**
   * Pite ke kadmu prvku matice konstantu.
   * 
   * @param k
   *          Konstanta.
   * @return Vrac referenci na sebe sama.
   */
  public Matrix addConst(int k)
  {
    for (int i = 0; i < this.m.length; i++)
    {
      for (int j = 0; j < this.m[i].length; j++)
      {
        this.m[i][j] += k;
      }
    }
    
    return this;
  }

  /**
   * Odete obsah zadan matice od aktuln matice.
   * 
   * @param m
   *          Odetan matice.
   * @return Vrac referenci na sebe sama.
   * @throws IncompatibleMatrixException
   *           Generuje vjimku v ppad, e matice nejsou rozmrov
   *           kompatibiln.
   */
  public Matrix sub(Matrix m)
      throws IncompatibleMatrixException
  {
    if (this.getRows() != m.getRows() || this.getCols() != m.getCols())
    {
      throw new IncompatibleMatrixException();
    }
    return sub(m.m);
  }

  /**
   * Odete obsah zadan matice od aktuln matice.
   * 
   * @param m
   *          Odetan matice.
   * @return Vrac referenci na sebe sama.
   */
  private Matrix sub(int[][] m)
  {
    assert this.m.length == m.length;
    
    for (int i = 0; i < this.m.length; i++)
    {
      assert this.m[i].length == m[i].length;
      
      for (int j = 0; j < this.m[i].length; j++)
      {
        this.m[i][j] -= m[i][j];
      }
    }

    return this;
  }

  /**
   * Vynsob matici konstantou.
   * 
   * @param k
   *          Multiplikan konstanta.
   * @return Vrac referenci na sebe sama.
   */
  public Matrix multConst(int k)
  {
    for (int i = 0; i < m.length; i++)
    {
      for (int j = 0; j < m[i].length; j++)
      {
        m[i][j] *= k;
      }
    }

    return this;
  }

  /**
   * V ppad, e tato operace m smysl, vynsob dv matice (a * b) a vrt
   * vsledn souin jako novou matici (c).
   * 
   * @param a
   *          Matice na prvnm mst.
   * @param b
   *          Matice na druhm mst.
   * @return Vrac referenci na novou matici.
   * @throws IncompatibleMatrixException
   *           Generuje vjimku, pokud operace nsoben nem na zadanch
   *           maticch smysl.
   */
  public static Matrix getProduct(Matrix a, Matrix b)
      throws IncompatibleMatrixException
  {
    if (a.getCols() != b.getRows())
      throw new IncompatibleMatrixException();

    Matrix c = new Matrix(a.getRows(), b.getCols());
    for (int i = 0; i < a.getRows(); i++)
    {
      for (int j = 0; j < b.getCols(); j++)
      {
        for (int k = 0; k < a.getCols(); k++)
        {
          c.m[i][j] += a.m[i][k] * b.m[k][j];
        }
      }
    }

    return c;
  }

  /**
   * Vynsob aktuln matici zadanou matic a vrt vsledek jako novou matici.
   * Aktuln i zadan matice zstanou neporueny. Provede se tato operace: c =
   * this * x, piem c je nvratov hodnota.
   * 
   * @param x
   *          Matice na druhm mst.
   * @return Vrac referenci na novou matici.
   * @throws IncompatibleMatrixException
   *           Generuje vjimku, pokud operace nsoben nem na danch maticch
   *           smysl.
   */
  public Matrix mult(Matrix x)
      throws IncompatibleMatrixException
  {
    return getProduct(this, x);
  }

  /**
   * Provede transpozici matice.
   * 
   * @return Vrac referenci na sebe sama.
   */
  public Matrix transpose()
  {
    int[][] tmp = m;
    int rows = getCols();
    int cols = getRows();
    create(rows, cols);

    for (int i = 0; i < rows; i++)
    {
      for (int j = 0; j < cols; j++)
      {
        m[i][j] = tmp[j][i];
      }
    }

    return this;
  }

  /**
   * Vytiskne matici na zadan vstupn proud.
   * 
   * @param stream
   *          Vstupn proud, do kterho se bude tisknout obsah matice.
   */
  public void print(PrintStream stream)
  {
    for (int i = 0; i < m.length; i++)
    {
      for (int j = 0; j < m[i].length; j++)
      {
        stream.printf("%3d ", m[i][j]);
      }
      stream.println();
    }
    stream.println();
  }

  /**
   * Klonovac metoda - vytvo identickou kopii objektu. Tato metoda pekrv
   * implicitn implementaci ze tdy <code>Object</code>. Aby tato metoda
   * fugovala sprvn, je nutn, aby tato tda implementovala rozhran
   * <code>Cloneable</code>. Dle je dleit, aby vytvoila skuten
   * hlubokou kopii tohoto objektu. Kdyby vytvela pouze mlkou kopii (tj.
   * koprovala by napklad reference, msto alokace novch objekt), velmi
   * pravdpodobn by to zpsobovalo zvan a zludn chyby.
   */
  public Object clone()
  {
    try
    {
      Matrix mat = (Matrix) super.clone();
      mat.create(this.getRows(), this.getCols());
      mat.setValue(this.m);
      return mat;
    }
    catch (CloneNotSupportedException e)
    {
      throw new InternalError(
          "Tohle by nemlo nastat! Nadtda Matrix nepodporuje klonovn!");
    }
  }

  /**
   * <p>
   * Porovn dva objekty. Bhem porovnvn provd porovnn prvek po prvku.
   * Pokud chce programtor v Jav porovnvat dva objekty, neme k tomu pout
   * opertor ==, protoe ten porovnv pouze reference (tedy adresy) a ne
   * skuten obsah objekt.
   * </p>
   * <p>
   * Tato metoda je zddn ze tdy <code>Object</code>.
   * </p>
   * 
   * @see java.lang.Object#equals(java.lang.Object)
   */
  public boolean equals(Object obj)
  {
    if (obj instanceof Matrix)
    {
      Matrix mat = (Matrix) obj;
      if (mat.getCols() != this.getCols() || mat.getRows() != this.getRows())
        return false;

      for (int i = 0; i < m.length; i++)
      {
        for (int j = 0; j < m[i].length; j++)
        {
          if (m[i][j] != mat.m[i][j])
            return false;
        }
      }
      return true;
    }
    else
      return false;
  }

  /**
   * Metoda main zde slou pouze pro testovn. Pro vslednou aplikaci nem
   * vznam.
   */
  public static void main(String[] args)
  {
    try
    {
      Matrix m = new Matrix(4, 4);
      System.out.printf("%d x %d%n", m.getRows(), m.getCols());
      Matrix n = (Matrix) m.clone();
      System.out.printf("%d x %d%n", n.getRows(), n.getCols());
      n.print(System.out);
      m.print(System.out);

      Matrix o = new Matrix(new int[][] { { 1, 1, 1 }, { 1, 1, 1 } });
      o.print(System.out);
      Matrix p = new Matrix(2, 4);
      p.setValue(new int[][] { { 2, 2, 2, 1 }, { 2, 2, 2, 1 } });
      p.print(System.out);
      // o.add(p);
      // o.print(System.out);
      p.multConst(2);
      p.print(System.out);
      Matrix E = Matrix.createEMatrix(3);
      E.print(System.out);
      Matrix R = Matrix.createRandomMatrix(3, 2);
      R.print(System.out);
      o.transpose();
      o.print(System.out);
      Matrix.getProduct(o, p).print(System.out);
    }
    catch (IncompatibleMatrixException e)
    {
      e.printStackTrace();
    }
  }
}

// end of file Matrix.java
