/*
 * Project: Lekce04
 * Author: David Martinek
 * e-mail: martinek (@) fit.vutbr.cz
 *
 * Created on 19.3.2005
 */
package dm.vpi.lekce04.matrix;

import junit.framework.TestCase;

/**
 * <p>
 * Testovac tda pro tdu Matrix, kter vyuv knihovnu JUnit. Tato knihovna
 * poskytuje monost vytvet automatizovan testy nad vyvjenou aplikac. Jde o
 * velmi uiten a vyuvan nstroj. Poskytuje toti velkou mru jistoty, e
 * jakkoli pozdj prava aplikace nebude mt nepedvdan vedlej efekty.
 * </p>
 * <p>
 * Tato tda mus bt potomkem tdy junit.framework.TestCase. Promnn
 * vyuvan vemi testy se nastavuj v metod <code>setUp()</code>.
 * Jednotliv testy se vytv jako metody <code>testNco()</code>, kde Nco
 * je nzev testu - pedpona <code>test</code> je dleit.
 * </p>
 * 
 * @author "David Martinek"
 */
public class MatrixTest
    extends TestCase
{
  private Matrix E;

  private Matrix Zero;

  private Matrix Zero5x5;

  private Matrix One5x5;

  private Matrix Zero2x5;

  private Matrix One5x2;

  private Matrix Rand5x2;

  private Matrix Rand5x5;

  private Matrix Zero5x2;

  private Matrix Zero2x0;

  protected void setUp()
      throws Exception
  {
    E = Matrix.createEMatrix(5);
    Zero = new Matrix(0, 0);

    Zero5x5 = new Matrix(5, 5);
    Zero2x5 = new Matrix(2, 5);
    Zero5x2 = new Matrix(5, 2);
    Zero2x0 = new Matrix(2, 0);
    Rand5x2 = Matrix.createRandomMatrix(5, 2);
    Rand5x5 = Matrix.createRandomMatrix(5, 5);
    One5x2 = new Matrix(5, 2).addConst(1);
    One5x5 = new Matrix(5, 5).addConst(1);
  }

  /*
   * Class under test for void Matrix(int, int)
   */
  public void testMatrixintint()
  {
    assertEquals("Przdn matice 5x5", Zero5x5, new Matrix(5, 5));
    assertEquals("Przdn matice 2x5", Zero2x5, new Matrix(2, 5));
    assertEquals("Matice 0x0", Zero, new Matrix(0, 0));
    assertEquals("Matice 2x0", Zero2x0, new Matrix(2, 0));
  }

  /*
   * Class under test for Object clone()
   */
  public void testClone()
  {
    assertEquals("Kopie 0x0", Zero, Zero.clone());
    assertEquals("Kopie przdn 5x5", Zero5x5, Zero5x5.clone());
    assertEquals("Kopie przdn 2x5", Zero2x5, Zero2x5.clone());
    assertEquals("Kopie przdn 2x0", Zero2x0, Zero2x0.clone());
    Matrix mat = (Matrix) Rand5x2.clone();
    assertEquals("Kopie nhodn 5x2", Rand5x2, mat);
    assertNotSame("Test hlubok kopie", Rand5x2.m[0], mat.m[0]);
    mat.m[1][1] += 10;
    assertFalse("Klonovan matice po zmn", Rand5x2.equals(mat));
  }

  public void testCreateEMatrix()
  {
    assertEquals(E, new Matrix(new int[][] { { 1, 0, 0, 0, 0 },
        { 0, 1, 0, 0, 0 }, { 0, 0, 1, 0, 0 }, { 0, 0, 0, 1, 0 },
        { 0, 0, 0, 0, 1 } }));
  }

  public void testCreateRandomMatrix()
  {
    assertFalse(Rand5x2.equals(Matrix.createRandomMatrix(5, 2)));
  }

  public void testGetRows()
  {
    assertEquals(Zero2x0.getRows(), 2);
    assertEquals(Zero.getRows(), 0);
    assertEquals(Rand5x2.getRows(), 5);
  }

  public void testGetCols()
  {
    assertEquals(Zero2x0.getCols(), 0);
    assertEquals(Zero.getCols(), 0);
    assertEquals(Rand5x2.getCols(), 2);
  }

  /*
   * Class under test for Matrix setValue(Matrix)
   */
  public void testSetValueMatrix()
  {
    try
    {
      Matrix tmp = One5x2.setValueOf(Rand5x2);
      assertEquals(tmp, Rand5x2);
      assertNotSame(One5x2, Rand5x2);
      assertSame(tmp, One5x2);
    }
    catch (IncompatibleMatrixException e)
    {
      fail("Tyhle matice by mly mt stejn rozmry.");
    }

  }

  /*
   * Class under test for Matrix add(Matrix)
   */
  public void testAddMatrix()
  {
    try
    {
      Matrix tmp = Zero5x2.add(Rand5x2);
      assertEquals("Souet Nulov a nhodn = nhodn", tmp, Rand5x2);
      assertNotSame(One5x2, Rand5x2);
      assertSame(tmp, Zero5x2);
      tmp = (Matrix) Rand5x2.clone();
      tmp.add(tmp);
      Rand5x2.multConst(2);
      assertEquals("M+M == M*2", tmp, Rand5x2);
      assertNotSame(tmp, Rand5x2);

      tmp = (Matrix) Zero.clone();
      tmp.add(Zero);
      assertEquals("0+0=0", tmp, Zero);

      tmp = (Matrix) Zero2x0.clone();
      tmp.add(Zero2x0);
      assertEquals("0+0=0", tmp, Zero2x0);
    }
    catch (IncompatibleMatrixException e)
    {
      fail("Tyhle matice by mly mt stejn rozmry.");
    }
    try
    {
      Rand5x2.add(One5x5);
      fail("Tento kd by neml bt dosaen.");
    }
    catch (IncompatibleMatrixException e)
    {
      // ok
    }
  }

  /*
   * Class under test for Matrix sub(Matrix)
   */
  public void testSubMatrix()
  {
    try
    {
      Matrix tmp = (Matrix) Rand5x2.clone();
      tmp.add(One5x2).sub(One5x2);
      assertEquals("A+1-1 = A", tmp, Rand5x2);
      tmp.sub(One5x2).multConst(2).sub(One5x2);
      Rand5x2.multConst(2).sub(One5x2.multConst(3));
      assertEquals("(A-1)*2-1 == 2*A-3", tmp, Rand5x2);

      tmp = (Matrix) Zero.clone();
      tmp.sub(Zero);
      assertEquals("0+0=0", tmp, Zero);

      tmp = (Matrix) Zero2x0.clone();
      tmp.sub(Zero2x0);
      assertEquals("0+0=0", tmp, Zero2x0);
    }
    catch (IncompatibleMatrixException e)
    {
      fail("Tyhle matice by mly mt stejn rozmry.");
    }
    try
    {
      Rand5x2.sub(One5x5);
      fail("Tento kd by neml bt dosaen.");
    }
    catch (IncompatibleMatrixException e)
    {
      // ok
    }
  }

  /*
   * Class under test for Matrix AddConst(int)
   */
  public void testAddConstint()
  {
    assertEquals(One5x2, Zero5x2.addConst(1));
    assertEquals(Zero, Zero.addConst(3249));
    assertEquals(Zero2x0, Zero2x0.addConst(325));
  }

  /*
   * Class under test for Matrix multConst(int)
   */
  public void testMultConstint()
  {
    assertEquals(Zero2x5, Zero2x5.multConst(345));
    assertEquals(Zero, Zero.multConst(3249));
    assertEquals(Zero2x0, Zero2x0.multConst(325));
    Matrix tmp = (Matrix) One5x2.clone();
    assertEquals(One5x2.addConst(5), tmp.multConst(3).multConst(2));
  }

  public void testGetProduct()
  {
    try
    {
      Matrix A = Matrix.getProduct(E, Rand5x5);
      assertEquals(A, Rand5x5);
      
      A = (Matrix) Rand5x2.clone();
      A.transpose();
      Matrix B = Matrix.getProduct(A, Rand5x2);
      assertEquals(B.getCols(), 2);
      assertEquals(B.getRows(), 2);
    }
    catch (IncompatibleMatrixException e)
    {
      fail("Tyhle matice by mly bt kompatibiln.");
    }
    
    try
    {
      Matrix C = Matrix.getProduct(Rand5x2, Rand5x2);
      fail("Sem by se to nemlo dostat.");
    }
    catch (IncompatibleMatrixException e)
    {
      // ok
    }

  }

  /*
   * Class under test for Matrix mult(Matrix)
   */
  public void testMultMatrix()
  {
    try
    {
      assertEquals("0*M=0", Zero5x5.mult(Rand5x5), Zero5x5);
      assertEquals("M*0=0", Rand5x5.mult(Zero5x5), Zero5x5);
      
      Matrix A = new Matrix(new int[][]{
          {1,5,8,2},
          {1,7,0,4}
      });
      Matrix B = new Matrix(new int[][]{
          {1,8,6},
          {2,0,0},
          {6,2,0},
          {2,4,3}
      });
      Matrix C = new Matrix(new int[][]{
          {63,32,12},
          {23,24,18}
      });
      assertEquals(A.mult(B), C);
      
      A = new Matrix(new int[][]{
          {-1,5,8,2},
          {1,7,0,-4}
      });
      
      C = new Matrix(new int[][]{
          {61,16,0},
          {7,-8,-6}
      });
      assertEquals(A.mult(B), C);
      
      assertEquals(Zero.mult(Zero), Zero);
    }
    catch (IncompatibleMatrixException e)
    {
      fail("Matice by mly bt kompatibiln.");
    }
  }

  public void testTranspose()
  {
    Matrix A = new Matrix(new int[][]{
        {1,8,6},
        {2,0,0},
        {6,2,0},
        {2,4,3}
    });

    Matrix B = new Matrix(new int[][]{
        {1,2,6,2},
        {8,0,2,4},
        {6,0,0,3}
    });
    assertEquals(A.transpose(), B);

  }

}
